
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_level is
    Port ( Data_IN	: inout	Std_Logic_Vector (7 downto 0);				-- Daten von/an USB-Baustein
			  Data_OUT	: out		Std_Logic_Vector (7 downto 0) := X"00";	-- Daten an die DDS-Bausteine
           CLK			: in		Std_Logic;											-- Clock Eingang
			  CLK_DDS	: out		Std_Logic;											-- Clock Ausgang fr DDS
           RD			: out		Std_Logic;											-- toggle RD um Daten vom USB-Baustein zu lesen
           WR			: out		Std_Logic;											-- toggle WR um Daten an USB-Baustein zu senden
           TXE			: in		Std_Logic;											-- signalisiert das Daten in den USB-Baustein geschrieben werden knnen
           RXF			: in		Std_Logic;											-- signalisiert das Daten zum einlesen im USB-Baustein sind
           PWREN		: in		Std_Logic;
           W_CLK		: out		Std_Logic_Vector (3 downto 0) := X"0";		-- mit der W_CLK knnen Daten bestimmten DDS-Bausteinen zugewiesen werden
           RESET		: out		Std_Logic := '0';									-- Reset der DDS-Bausteine
           FQ_UD		: out		Std_Logic := '0');								-- mit FQ_UD werden alle DDS-Bausteine synchron gestartet
end top_level;

architecture Behavioral of top_level is

signal state					:	Std_Logic_Vector (3 downto 0);
signal next_state				:	Std_Logic_Vector (3 downto 0);
signal checksum				:	Std_Logic_Vector (7 downto 0) := X"00";
signal Z_enable				:	Std_Logic := '1';
signal data_IN_int			:	Std_Logic_Vector (7 downto 0);
signal DDS						:	Std_Logic_Vector (2 downto 0);
signal RD_int					:  Std_Logic;						
signal Data_OUT_int 			: 	Std_Logic_Vector (7 downto 0);

begin
RD <= RD_int ;
Data_OUT <= Data_OUT_int;
CLK_DDS <= CLK;
-- Tri-State
buffer_out : for i in 0 to 7 generate
   OBUFT_inst : OBUFT
   generic map (
      DRIVE => 12,
      IOSTANDARD => "DEFAULT",
      SLEW => "SLOW")
   port map (
      O => Data_IN(i),		-- Buffer output (connect directly to top-level port)
      I => checksum(i),		-- Buffer input
      T => Z_enable			-- 3-state enable input 
   );
	
IBUF_inst : IBUF
   generic map (
      IOSTANDARD => "DEFAULT")
   port map (
      O => Data_IN_int(i),	-- Buffer output
      I => Data_IN(i)		-- Buffer input (connect directly to top-level port)
   );
end generate;

-- Daten vom PC einlesen und an DDS-Bausteine bergeben
Status : process (CLK, RXF)
	begin
		if RXF = '1' then 
			state <= B"0000";
			RD_int <= '1';
		elsif rising_edge(CLK) then
			if next_state = B"0000" or next_state = B"0010" or next_state = B"0100" or next_state >= B"1100" then 
					RD_int <= '1'; 
			else	
					RD_int <= '0';
			
			end if;
			state <= next_state;
		end if;
	end process Status;

Daten : process (CLK)
	begin
		if falling_edge(CLK) then  
			if RD_int = '0' then
				Data_OUT_int (7 downto 0) <= Data_IN_int (7 downto 0);
			end if;
			if Data_IN_int = X"1F" then
				DDS <= B"001";
			elsif Data_IN_int = X"2F" then
				DDS <= B"010";
			elsif Data_IN_int = X"3F" then
				DDS <= B"011";
			elsif Data_IN_int = X"4F" then
				DDS <= B"100";
			elsif Data_IN_int = X"EF" then
				DDS <= B"000";
			end if;
		end if;
	end process Daten;
	
WORDCLOCK : process (CLK, RXF, DDS)
	begin
		if RXF = '1' then
			W_CLK (3 downto 0) <= X"0";
		elsif falling_edge (CLK) then
			if DDS = B"001" and state = B"100" and Data_OUT_int /= X"1F" then
				W_CLK (3 downto 0) <= X"1";
				checksum <= checksum + Data_OUT_int;
			elsif DDS = B"010" and state = B"100" and Data_OUT_int /= X"2F" then
				W_CLK (3 downto 0) <= X"2";
				checksum <= checksum + Data_OUT_int;
			elsif DDS = B"011" and state = B"100" and Data_OUT_int /= X"3F" then
				W_CLK (3 downto 0) <= X"4";
				checksum <= checksum + Data_OUT_int;
			elsif DDS = B"100" then
				if Data_IN_int = X"DF" or Data_IN_int = X"FF" then
					W_CLK (3 downto 0) <= X"0";
				elsif state = B"100" and Data_OUT_int /= X"4F" then
					W_CLK (3 downto 0) <= X"8";
					checksum <= checksum + Data_OUT_int;
				else W_CLK (3 downto 0) <= X"0";
				end if;
			else	W_CLK (3 downto 0) <= X"0";
			end if;
		end if;
	end process WORDCLOCK;

LabView_Daten : process (Data_IN_int, state, RXF)
	begin
		WR <= '0';
		RESET <= '0' ;
		FQ_UD <= '0';
		Z_enable <= '1';
		case state is
--Tristate entsprechend setzen
			when B"0000" =>	next_state <= B"0001";
--									FQ_UD <= '0';
									RESET <= '0';
			-- Synchrondaten "FF"
			when B"0001" =>		
									-- Synchrondatenwort DF fr Datenempfang
									if Data_IN_int = X"DF" and RXF = '0' then
										next_state <= B"0010";
									-- Synchrondatenwort FF fr FQ_UD
									elsif Data_IN_int = X"FF" and RXF = '0' then
										FQ_UD <= '1';
										next_state <= B"0101";
									-- Synchrondatenwort 0F fr RESET
									elsif Data_IN_int = X"0F" then
										RESET <= '1';
										next_state <= B"0110";
									-- Synchrondatenwort 9F fr Abgleich der CheckSum
									elsif Data_IN_int = X"9F" then
										next_state <= B"1011";
									-- ansonsten Rcksprung zum Anfang
									else
										next_state <= B"0000";
									end if;
			when B"0010" =>	next_state <= B"0011";
			--Daten	Einlesen fr DDS
			when B"0011" =>		-- falls direkt nach dem letzten Byte fr W_CLK 4 neue Daten kommen
									if Data_IN_int = X"DF" then 
										next_state <= B"0001";
									elsif Data_IN_int = X"FF" then
										next_state <= B"0101";
									elsif Data_IN_int = X"0F" then
										RESET <= '1';
										next_state <= B"0110";
									elsif Data_IN_int = X"1F" then
										next_state <= B"0100";
									elsif Data_IN_int = X"2F" then
										next_state <= B"0100";
									elsif Data_IN_int = X"3F" then
										next_state <= B"0100";										
									elsif Data_IN_int = X"4F" then
										next_state <= B"0100";										
									elsif Data_IN_int = X"EF" then
										next_state <= B"0010";
									elsif Data_IN_int = X"9F" then
										next_state <= B"1011";
									else
										next_state <= B"0100";
									end if;
			when B"0100" =>	if DDS /= B"0000" then
										next_state <= B"0011";
									else
										next_state <= B"0001";
									end if;
			when B"0101" =>	FQ_UD <= '1';
									next_state <= B"0000";
			when B"0110" =>	Reset <= '1';
									next_state <= B"0111";
			when B"0111" =>	Reset <= '1';
									next_state <= B"1000";
			when B"1000" =>	Reset <= '1';
									next_state <= B"1001";
			when B"1001" =>	Reset <= '1';
									next_state <= B"1010";
			when B"1010" =>	Reset <= '1';
									next_state <= B"0000";
			when B"1011" =>	next_state <= B"1100";
			when B"1100" =>	--RD <= '1';
									if TXE = '1' then
										next_state <= B"1011";
									elsif TXE = '0' then
										WR <= '1';
										next_state <= B"1101";
									else next_state <= B"0000";
									end if;
			when B"1101" =>	Z_enable <= '0';					-- legt CheckSum auf Data_IN
									WR <= '1';
									next_state <= B"1110";
			when B"1110" =>	WR <= '0';
									Z_enable <= '0';
									next_state <= B"1111";
			when others => 	next_state <= B"0000";
		end case;
	end process LabView_Daten;
	
end Behavioral;